# -*- coding: utf-8 -*-
"""Digital Forensics Virtual File System (dfVFS).

dfVFS, or Digital Forensics Virtual File System, is a Python module
that provides read-only access to file-system objects from various
storage media types and file formats.
"""
